<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Helper;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;

/**
 * Class Indexer
 * @package FiloBlu\Refilo\Helper
 */
class Indexer
{
    /** @var string */
    public const XML_SYNC_DATA_INDEX_BULK_SIZE = 'sync/data/index_bulk_size';

    /** @var string */
    public const XML_SYNC_DATA_ALLOW_INDEX_DELETION = 'sync/data/allow_index_deletion';

    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * Indexer constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(ResourceConnection $resourceConnection, ScopeConfigInterface $scopeConfig)
    {
        $this->resourceConnection = $resourceConnection;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @param $table
     * @param $idColumn
     * @return array
     */
    public function getDeletedEntityIds($table, $idColumn)
    {
        return $this->getDeletedEntityIdsByIntervals($this->getDeletedEntityIntervals($table, $idColumn));
    }

    /**
     * @param array $intervals array of [
     *      [ 'start' => 10, 'end' => 12 ],
     *      [ 'start' => 13, 'end' => 15 ]
     * ]
     * @return array
     */
    public function getDeletedEntityIdsByIntervals(array $intervals)
    {
        $ids = [];

        foreach ($intervals as $interval) {

            $start = $interval['start'];
            $end = $interval['end'];

            if (($end - $start) < 2) {
                continue;
            }

            $ids = array_merge($ids, range($start + 1, $end - 1));
        }

        return $ids;
    }

    /**
     * @param $table
     * @param $idColumn
     * @return array
     */
    public function getDeletedEntityIntervals($table, $idColumn)
    {
        $starts =
            $this->resourceConnection->getConnection()
                ->fetchAll("SELECT	l.{$idColumn} AS `start` FROM {$table} AS l LEFT OUTER JOIN {$table} AS r ON l.{$idColumn} = r.{$idColumn} -1 WHERE r.{$idColumn} IS NULL");

        $ends =
            $this->resourceConnection->getConnection()
                ->fetchAll(
                    "SELECT	l.{$idColumn} AS `end` FROM {$table} AS l LEFT OUTER JOIN {$table} AS r ON l.{$idColumn} = r.{$idColumn} +1 WHERE r.{$idColumn} IS NULL HAVING `end` > 1"
                );

        $intervals = [];

        foreach ($starts as $start) {
            $end = array_shift($ends);
            $intervals[] = [
                'start' => (int)$start['start'],
                'end' => (int)($end['end'] ?? $start['start'] + 1)
            ];
        }

        return $intervals;
    }

    /**
     * @return int
     */
    public function getIndexerBulkSize(): int
    {
        return (int)$this->scopeConfig->getValue(self::XML_SYNC_DATA_INDEX_BULK_SIZE) ?: 500;
    }

    /**
     * @return bool
     */
    public function indexerCanDelete(): bool
    {
        return (bool)$this->scopeConfig->isSetFlag(self::XML_SYNC_DATA_ALLOW_INDEX_DELETION);
    }

    /**
     *
     */
    public function fixPriceIndex()
    {
        $this->resourceConnection->getConnection()->query('UPDATE
	catalog_product_index_price AS q1,
	(
	SELECT
		MIN(cpp.rule_price) AS min_price,
		MIN(cpp.rule_price) AS final_price,
		cpp.website_id,
		cpp.customer_group_id,
		cpe.entity_id
	FROM
		catalog_product_entity cpe
	INNER JOIN catalog_product_entity_int cpei ON
		cpe.row_id = cpei.row_id
	INNER JOIN eav_attribute ea ON
		cpei.attribute_id = ea.attribute_id
	INNER JOIN catalog_product_super_link cpsl ON
		cpe.row_id = cpsl.parent_id
	INNER JOIN catalogrule_product_price cpp ON
		cpp.product_id = cpsl.product_id
	INNER JOIN catalog_product_entity cpe2 ON
		cpe2.entity_id = cpp.product_id
	WHERE
		cpe.type_id = \'configurable\'
		AND ea.attribute_code = \'status\'
	GROUP BY
		cpe.entity_id ,
		cpp.website_id ,
		cpp.customer_group_id ) AS q2 SET
	q1.min_price = q2.min_price,
	q1.final_price = q2.final_price
WHERE
	q1.website_id = q2.website_id
	AND q1.customer_group_id = q2.customer_group_id
	AND q1.entity_id = q2.entity_id');
    }
}
